program FindLinesContString;
{$APPTYPE CONSOLE}

uses
  SysUtils;

function StrIsParameter(S: String): Boolean;
var
	option: Boolean;
	i:Integer;
	T: String;
begin
	// if '-' or '/' is passed as first character of S, then search for both, as
	// either would be considered a "valid switch".
	Result := False;
	option := False;
	if ParamCount > 0 then begin
		if (((Copy(S,1,1) = '-') or (Copy(S,1,1) = '/')) and (Length(S) > 1)) then begin
			option := True;
			T := Copy(S,2,Length(S));
		end;
		for i := 1 to ParamCount do begin
			if option then begin
				if (ParamStr(i) = '-' + T) or (ParamStr(i) = '/' + T) then begin
					Result := True;
					Exit;
				end;
			end
			else begin
				if (ParamStr(i) = S) then begin
					Result := True;
					Exit;
				end;
			end;
		end;
	end;
end;

function StrIsParameterSubStr(S: String; var Remainder: String): Boolean;
var
	option: Boolean;
	i:Integer;
	T: String;
begin
	// if '-' or '/' is passed as first character of S, then search for both, as
	// either would be considered a "valid switch".
	Result := False;
	option := False;
	Remainder := '';
	if ParamCount > 0 then begin
		if (((Copy(S,1,1) = '-') or (Copy(S,1,1) = '/')) and (Length(S) > 1)) then begin
			option := True;
			T := Copy(S,2,Length(S));
		end;
		for i := 1 to ParamCount do begin
			if option then begin
				if (Pos('-' + T, ParamStr(i)) = 1) or (Pos('/' + T, ParamStr(i)) = 1) then begin
					Result := True;
					Remainder := Copy(ParamStr(i), Length(S) + 1, Length(ParamStr(i)));
					Exit;
				end;
			end
			else begin
				if (Pos(S, ParamStr(i)) = 1) then begin
					Result := True;
					Remainder := Copy(ParamStr(i), Length(S) + 1, Length(ParamStr(i)));
					Exit;
				end;
			end;
		end;
	end;
end;

function ValidFilenameParam(var Filename: String): Boolean;
var
	i: Integer;
begin
	Filename := '';
	Result := False;
	if ParamCount > 0 then begin
		for i := 1 to ParamCount do begin
			if FileExists(ParamStr(i)) then begin
				Result := True;
				Filename := ParamStr(i);
				Exit;
			end;
		end;
	end;
end;

var
	S, M, SLine, TLine, PassedFilename: String;
	F: TextFile;
	ShowHelp: Boolean;
	V, C, N, I: Boolean;
	Count, LineNumber: Int64;
begin
  try
  	// Initialize variables to default values.
  	ShowHelp := False;
  	V := False;
  	C := False;
  	N := False;
  	I := False;
  	Count := 0;
  	LineNumber := 0;

  	// check to see if a valid filename was passed, and retrieve it.
		if ValidFilenameParam(PassedFilename) then begin
			// We have a filename, now check for other options and for the string!
			// Check for string first, without which we will exit to the help.
			if StrIsParameterSubStr('"', S) then begin
				M := S;
				// Extract the String
				if Copy(S, Length(S), 1) = '"' then S := Copy(S, 1, Length(S) - 1);
				// Great! We have a string!  Now let's check other options.
				if StrIsParameter('-V') or StrIsParameter('-v') then V := True;
				if StrIsParameter('-C') or StrIsParameter('-c') then C := True;
				if StrIsParameter('-N') or StrIsParameter('-n') then N := True;
				if StrIsParameter('-I') or StrIsParameter('-i') then I := True;
				if I then S := UpperCase(S);
				AssignFile(F, PassedFilename);
				Reset(F);

				// Always enclose file operations inside a try/finally statement, so
				// that the files can be closed in the "finally" part, and will always
				// be closed, even in case of error that stops the program.
				try

					// This starts the actual code that parses the file and generates
					// the output.
					while not Eof(F) do begin
						Readln(F, SLine);
						if I then TLine := UpperCase(SLine) // If we're doing case INsensitive
						else TLine := SLine; // if we're doing case sensitive
						Inc(LineNumber); // LineNumber count in case we're outputting line numbers with the line.
						if I then
						if Pos(S, TLine) > 0 then begin // If we found the string, then do the following.
							Inc(Count); // In case "Count of lines containing string" switch is specified.
							if not C then begin // If we're not doing a count only, then we need to display a line.
								if N then SLine := '(' + IntToStr(LineNumber) + ')' + SLine; // Add the line number if switch was specified.
								if (not V) then Writeln(SLine); // Display the line where string was found if the -V switch was not specified.
							end;
						end
						else begin // If we didnt' find the string, then do the following.
							if not C then begin // If we're not doing a count only, then we need to display a line.
								if N then SLine := '(' + IntToStr(LineNumber) + ')' + SLine; // Add the line number if switch was specified.
								if V then Writeln(SLine); // Display any line where the string was NOT found if -V was specified.
							end;
						end;
					end;
					// This ENDS the actual code that parses the file and generates
					// the output.

				finally
					CloseFile(F);
				end;
				// End of File reached!  Show the count if you're supposed to!
				if C then Writeln('Found "' + M + '" in ' + IntToStr(Count) + ' lines.');
			end
			else ShowHelp := True; // No "string" passed, so show the help.
		end
		else ShowHelp := True; // No valid filename passed, so show the help.
		if ShowHelp then begin
			// Show the Help text.
      Writeln('FindLinesContString version 1.0');
      Writeln('Copyright 2012 by Daniel W. Moore Development.');
      Writeln('This program was written to get around the limitations of the Find command,');
      Writeln('which could not handle files containing over 2,147,483,647 lines.');
      Writeln('This program can handle a file up to 9,223,372,036,854,779,999 lines.');
      Writeln('');
      Writeln('USAGE: FindLinesContString [-V] [-C] [-N] [-I] "string" [[drive:][path]filename]');
      Writeln('');
      Writeln('-V  Displays all lines NOT containing the specified string.');
      Writeln('-C  Displays only the count of lines containing the string.');
      Writeln('-N  Displays line numbers with the displayed lines.');
      Writeln('-I  Ignores the case of characters when searching for the string.');
      Writeln('"string"  Specifies the text string to find. No regular expressions.');
      Writeln('[drive:][path]filename');
      Writeln('    Specifies a file to search.  No wildcard characters allowed.');
      Writeln('');
      Writeln('If a file is not specified, FindLinesContString displays this help.');
      Writeln('"-" and "/" are both considered valid switches. ');
      Writeln('Therefore using either "-V" or "/V" would be accepted.');
    end;
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
